//
//  OCRController.m
//  AsReaderScan
//
//  Created by Mikiko on 11/7/24.
//  Copyright © 2024 SPS INC. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "OCRController.h"

@interface OCRController () <UITableViewDataSource, UITableViewDelegate>

@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) UIButton *okButton;
@property (nonatomic, strong) UIButton *cancelButton;

@end

@implementation OCRController

- (instancetype)init {
    self = [super init];
    if (self) {
        _options = @[@"OCRA", @"OCRB"];
        _selectedOptions = [NSMutableSet set];
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [self setupUI];
}

- (void)setupUI {
    self.view.backgroundColor = [UIColor whiteColor];
    
    self.tableView = [[UITableView alloc] initWithFrame:CGRectZero style:UITableViewStylePlain];
    self.tableView.translatesAutoresizingMaskIntoConstraints = NO;
    self.tableView.dataSource = self;
    self.tableView.delegate = self;
    [self.tableView registerClass:[UITableViewCell class] forCellReuseIdentifier:@"OptionCell"];
    [self.view addSubview:self.tableView];
    
    self.okButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.okButton setTitle:@"Save" forState:UIControlStateNormal];
    [self.okButton addTarget:self action:@selector(okButtonTapped) forControlEvents:UIControlEventTouchUpInside];
    self.okButton.translatesAutoresizingMaskIntoConstraints = NO;
    [self.view addSubview:self.okButton];
    
    self.cancelButton = [UIButton buttonWithType:UIButtonTypeSystem];
    [self.cancelButton setTitle:@"Cancel" forState:UIControlStateNormal];
    [self.cancelButton addTarget:self action:@selector(cancelButtonTapped) forControlEvents:UIControlEventTouchUpInside];
    self.cancelButton.translatesAutoresizingMaskIntoConstraints = NO;
    [self.view addSubview:self.cancelButton];

    [NSLayoutConstraint activateConstraints:@[
        [self.tableView.topAnchor constraintEqualToAnchor:self.view.topAnchor constant:20],
        [self.tableView.leadingAnchor constraintEqualToAnchor:self.view.leadingAnchor constant:20],
        [self.tableView.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor constant:-20],
        [self.tableView.heightAnchor constraintEqualToConstant:150],
        
        [self.okButton.topAnchor constraintEqualToAnchor:self.tableView.bottomAnchor constant:20],
        [self.okButton.leadingAnchor constraintEqualToAnchor:self.view.leadingAnchor constant:20],
        [self.okButton.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor constant:-20],
        [self.okButton.heightAnchor constraintEqualToConstant:40],
        
        [self.cancelButton.topAnchor constraintEqualToAnchor:self.okButton.bottomAnchor constant:10],
        [self.cancelButton.leadingAnchor constraintEqualToAnchor:self.view.leadingAnchor constant:20],
        [self.cancelButton.trailingAnchor constraintEqualToAnchor:self.view.trailingAnchor constant:-20],
        [self.cancelButton.heightAnchor constraintEqualToConstant:40],
    ]];
}

- (void)setDataWithList:(NSArray<NSString *> *)list selectList:(NSArray<NSString *> *)selectList {
    self.options = list;

    for (NSString *message in selectList) {
        if ([message isEqualToString:@"OCRA"]) {
            [self.selectedOptions addObject:self.options[0]];
        }
        
        if ([message isEqualToString:@"OCRB"]) {
            [self.selectedOptions addObject:self.options[1]];
        }
    }
}

- (void)okButtonTapped {
    if ([self.delegate respondsToSelector:@selector(didSelectOptions:)]) {
        [self.delegate didSelectOptions:self.selectedOptions];
    }
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)cancelButtonTapped {
    [self dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - UITableViewDataSource & UITableViewDelegate

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.options.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"OptionCell" forIndexPath:indexPath];
    NSString *option = self.options[indexPath.row];
    cell.textLabel.text = option;
    
    if ([self.selectedOptions containsObject:option]) {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    } else {
        cell.accessoryType = UITableViewCellAccessoryNone;
    }

    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    NSString *selectedOption = self.options[indexPath.row];

    if ([self.selectedOptions containsObject:selectedOption]) {
        [self.selectedOptions removeObject:selectedOption];
    } else {
        [self.selectedOptions addObject:selectedOption];
    }

    [tableView reloadRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationAutomatic];
}

@end
