//
//  SettingsCommonController.m
//  AsReaderScan
//
//  Created by Mikiko on 11/7/24.
//  Copyright © 2024 SPS INC. All rights reserved.
//

#import "SettingsCommonController.h"
#import "ModelSettings.h"
#import "Utils.h"
#import <AsReaderDockSDK/AsReaderDockSDK.h>

@interface SettingsCommonController ()<UITextFieldDelegate, AsreaderBarcodeDeviceDelegate,AsReaderDeviceDelegate>
{
    BOOL mIsBeep;
    BOOL mIsVib;
    BOOL mIsLed;
    BOOL mIsIllu;
}

#define KEY_SETTING_SYMBOLOGY_PREFIX_ENABLE @"keySettingSymbologyPrefixEnable"
#define KEY_SETTING_CUSTOM_PREFIX_ENABLE @"keySettingCustomPrefixEnable"
#define KEY_SETTING_CUSTOM_SUFFIX_ENABLE @"keySettingCustomSuffixEnable"
#define KEY_SETTING_CUSTOM_PREFIX_VALUE @"keySettingCustomPrefixValue"
#define KEY_SETTING_CUSTOM_SUFFIX_VALUE @"keySettingCustomSuffixValue"

@property (weak, nonatomic) IBOutlet UISwitch *olBeep;
@property (weak, nonatomic) IBOutlet UISwitch *olIllumination;
@property (weak, nonatomic) IBOutlet UISwitch *olVibration;
@property (weak, nonatomic) IBOutlet UISwitch *olLED;
@property (weak, nonatomic) IBOutlet UISwitch *olAutoPower;
@property (weak, nonatomic) IBOutlet UISwitch *olDefaultTriggerOn;
@property (weak, nonatomic) IBOutlet UISwitch *olPowerOnBeep;
@property (weak, nonatomic) IBOutlet UISwitch *olChargingControl;
@property (weak, nonatomic) IBOutlet UITextField *olUserCommand;

@property(weak, nonatomic) IBOutlet UISwitch *customPrefixSwitch;
@property(weak, nonatomic) IBOutlet UISwitch *customSuffixSwitch;
@property(weak, nonatomic) IBOutlet UISwitch *symbologyPrefixSwitch;
@property(weak, nonatomic) IBOutlet UITextField *customPrefixField;
@property(weak, nonatomic) IBOutlet UITextField *customSuffixField;

/*20190909*/
@property  (weak,nonatomic) IBOutlet UITableViewCell *olCellFactory;
@property  (weak,nonatomic) IBOutlet UITableViewCell *olCellChargingControl;
@property  (weak,nonatomic) IBOutlet UITableViewCell *olCellDefaultTrigger;
@property  (weak,nonatomic) IBOutlet UITableViewCell *olCellBarcodeUserCommand;

@property(weak, nonatomic) IBOutlet UITableViewCell *customPrefixCell;
@property(weak, nonatomic) IBOutlet UITableViewCell *customSuffixCell;
@property(weak, nonatomic) IBOutlet UITableViewCell *symbologyPrefixCell;
@property(weak,nonatomic) IBOutlet UITableViewCell *olCellSleepTime;
@end

@implementation SettingsCommonController

#pragma mark - Lifecycle
//----------------------------------------------------------------
- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self showHideCell];
    
    self.olUserCommand.delegate = self;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:YES];
    [AsReaderBarcodeDevice sharedInstance].delegateBarcode = self;
    [AsReaderRFIDDevice sharedInstance].delegateDevice = self;
    
    BOOL isAutoPower = [[NSUserDefaults standardUserDefaults] boolForKey:@"AutoPowerOn"];
    BOOL isDefaultTriggerOn = [[NSUserDefaults standardUserDefaults] boolForKey:@"DefaultTriggerOn"];
    BOOL isBeepOn = [[NSUserDefaults standardUserDefaults] boolForKey:@"beep"];
    BOOL isVirbOn = [[NSUserDefaults standardUserDefaults] boolForKey:@"vibration"];
    BOOL isLedOn = [[NSUserDefaults standardUserDefaults] boolForKey:@"led"];
    BOOL isIllumOn = [[NSUserDefaults standardUserDefaults] boolForKey:@"illumination"];
    BOOL isPowerOnBeep =[[NSUserDefaults standardUserDefaults] boolForKey:@"powerOnBeep"];
    
    BOOL enableCustomPrefix = [[NSUserDefaults standardUserDefaults] boolForKey:KEY_SETTING_CUSTOM_PREFIX_ENABLE];
    BOOL enableCustomSuffix = [[NSUserDefaults standardUserDefaults] boolForKey:KEY_SETTING_CUSTOM_SUFFIX_ENABLE];
    NSString *customPrefixValue = [[NSUserDefaults standardUserDefaults] objectForKey:KEY_SETTING_CUSTOM_PREFIX_VALUE];
    NSString *customSuffixValue = [[NSUserDefaults standardUserDefaults] objectForKey:KEY_SETTING_CUSTOM_SUFFIX_VALUE];
    BOOL enableSymbologyPrefix = [[NSUserDefaults standardUserDefaults] boolForKey:KEY_SETTING_SYMBOLOGY_PREFIX_ENABLE];
    
    [self.customPrefixSwitch setOn:enableCustomPrefix];
    [self.customSuffixSwitch setOn:enableCustomSuffix];
    if( customPrefixValue==nil || [customPrefixValue isEqualToString:@""]){
        self.customPrefixField.text = @"01";
    } else {
        self.customPrefixField.text = customPrefixValue;
    }
    
    if( customSuffixValue==nil || [customSuffixValue isEqualToString:@""]){
        self.customSuffixField.text = @"01";
    } else {
        self.customSuffixField.text = customSuffixValue;
    }
    
    [self.symbologyPrefixSwitch setOn:enableSymbologyPrefix];
    
    [self.olPowerOnBeep setOn:isPowerOnBeep];
    [self.olBeep setOn:isBeepOn];
    [self.olVibration setOn:isVirbOn];
    [self.olLED setOn:isLedOn];
    [self.olIllumination setOn:isIllumOn];
    [self.olDefaultTriggerOn setOn:isDefaultTriggerOn];
    [self.olAutoPower setOn:isAutoPower];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:YES];
    [AsReaderBarcodeDevice sharedInstance].delegateBarcode = nil;
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

#pragma mark - Private method
//----------------------------------------------------------------
- (void)getDeviceInfo
{
    //Common
    [[AsReaderBarcodeDevice sharedInstance] getReaderInfo:00];
}

- (void)saveSettingVal
{
    AsReaderInfo *info = [AsReaderInfo sharedInstance];
    NSString *modelStr = info.deviceModelNumber;
    if ([modelStr isEqualToString:@"ASR-M24D"]) {
        [[AsReaderCommonDevice sharedInstance]
         setBeep:self.olBeep.isOn
         vibration:self.olVibration.isOn
         batteryGaugeLed:self.olLED.isOn
         barcodeAimer:self.olIllumination.isOn
         barcodePowerOnBeep:self.olPowerOnBeep.isOn];
    } else {
        [[AsReaderCommonDevice sharedInstance]
         setBeep:self.olBeep.isOn
         vibration:self.olVibration.isOn
         batteryGaugeLed:self.olLED.isOn
         barcodeAimer:self.olIllumination.isOn];
    }
    [[NSUserDefaults standardUserDefaults] setBool:self.olBeep.isOn forKey:@"beep"];
    [[NSUserDefaults standardUserDefaults] setBool:self.olIllumination.isOn forKey:@"illumination"];
    [[NSUserDefaults standardUserDefaults] setBool:self.olVibration.isOn forKey:@"vibration"];
    [[NSUserDefaults standardUserDefaults] setBool:self.olLED.isOn forKey:@"led"];
    [[NSUserDefaults standardUserDefaults] setBool:self.olPowerOnBeep.isOn forKey:@"powerOnBeep"];
        
    [[NSUserDefaults standardUserDefaults] setBool:self.customPrefixSwitch.isOn forKey:KEY_SETTING_CUSTOM_PREFIX_ENABLE];
    [[NSUserDefaults standardUserDefaults] setBool:self.customSuffixSwitch.isOn forKey:KEY_SETTING_CUSTOM_SUFFIX_ENABLE];
    [[NSUserDefaults standardUserDefaults] setObject:self.customPrefixField.text forKey:KEY_SETTING_CUSTOM_PREFIX_VALUE];
    [[NSUserDefaults standardUserDefaults] setObject:self.customSuffixField.text forKey:KEY_SETTING_CUSTOM_SUFFIX_VALUE];
    [[NSUserDefaults standardUserDefaults] setBool:self.symbologyPrefixSwitch.isOn forKey:KEY_SETTING_SYMBOLOGY_PREFIX_ENABLE];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

-(void)showHideCell
{
    AsReaderInfo *info = [AsReaderInfo sharedInstance];
    NSString *strModel = info.deviceModelNumber;
    
    NSString *hardwareVersionStr = [info.deviceHardware substringToIndex:1];
    int hardwareVersionInt = [hardwareVersionStr intValue];
 
    if([strModel containsString:@"A3XDB"]
       ||[strModel containsString:@"034B"]){
        [[AsReaderRFIDDevice sharedInstance] getSleepTimeForBLEDevice];
        _olCellSleepTime.hidden = NO;
    }else{
        _olCellSleepTime.hidden = YES;
    }
    
    if ([strModel isEqualToString:@"ASR-M24D"]) {
        NSLog(@"AutoPowerOn");
        _olAutoPower.enabled = NO;
    } else {
        _olAutoPower.enabled = YES;
    }
    
    //Charging control : 010D,010Dv2,020D,020Dv2,022D, 03xD,03xDv2,023xDv2 , 023xDv4
    if(!([strModel isEqualToString:@"ASR-010D"] ||
         [strModel isEqualToString:@"ASR-010D-V2"] ||
         [strModel isEqualToString:@"ASR-020D"] ||
         [strModel isEqualToString:@"ASR-020D-V2"] ||
         [strModel isEqualToString:@"ASR-022D"] ||
         [strModel isEqualToString:@"ASR-022D-V2U"] ||
         [strModel isEqualToString:@"ASR-030D"] ||
         [strModel isEqualToString:@"ASR-030D-V3"] ||
         [strModel isEqualToString:@"ASR-031D"] ||
         [strModel containsString:@"ASR-034B"] ||
         (([strModel containsString:@"ASR-023"]||[strModel containsString:@"ASR-024"])
          && hardwareVersionInt  >1))) {
        NSLog(@"ChargingControl");
        _olCellChargingControl.hidden = YES;
    }
    //Default Trigger control : 022D,03xD,03xDv2,023xD,023xDv2,0240D
    if(!([strModel isEqualToString:@"ASR-0230D"]
         ||[strModel isEqualToString:@"ASR-0231D"]
         ||[strModel isEqualToString:@"ASR-0240D"]
         ||[strModel isEqualToString:@"ASR-022D"]
         ||[strModel isEqualToString:@"ASR-022D-V2U"]
         ||[strModel isEqualToString:@"ASR-030D"]
         ||[strModel isEqualToString:@"ASR-030D-V3"]
         ||[strModel isEqualToString:@"ASR-031D"]
         ||[strModel containsString:@"034B"]
         ||[strModel containsString:@"ASR-023"])) {
        NSLog(@"DefaultTrigger");
        _olCellDefaultTrigger.hidden = YES;
    }
    
    //Symbology Prefix
    //Factory Reset
    //User Command
    //022D,023xD,0240D
    if(!([strModel isEqualToString:@"ASR-0230D"]
         ||[strModel isEqualToString:@"ASR-0231D"]
         ||[strModel isEqualToString:@"ASR-0240D"]
         ||[strModel isEqualToString:@"ASR-022D"]
         ||[strModel isEqualToString:@"ASR-022D-V2U"]
         ||[strModel containsString:@"ASR-023"])) {
        self.customPrefixCell.hidden = YES;
        self.customSuffixCell.hidden = YES;
        self.symbologyPrefixCell.hidden = YES;
        _olCellFactory.hidden = YES;
        _olCellBarcodeUserCommand.hidden = YES;
        
    } else {
        if( info.currentReaderMode==ReaderModeRfid ) {
            self.customPrefixCell.hidden = YES;
            self.customSuffixCell.hidden = YES;
            self.symbologyPrefixCell.hidden = YES;
            _olCellFactory.hidden = YES;
            _olCellBarcodeUserCommand.hidden = YES;
            
        } else if( info.currentReaderMode==ReaderModeNfc ) {
            self.customPrefixCell.hidden = YES;
            self.customSuffixCell.hidden = YES;
            self.symbologyPrefixCell.hidden = YES;
            _olCellFactory.hidden = YES;
            _olCellBarcodeUserCommand.hidden = YES;
        }
    }
}

- (void)showAlert:(NSString *)strMsg
{
    [Utils showAlert:@"" message:strMsg view:self];
}

#pragma mark - IBAction
//----------------------------------------------------------------





- (IBAction)eChargingControl:(id)sender
{
    UISwitch *sw = (UISwitch*)sender;
    [[AsReaderBarcodeDevice sharedInstance] setChargingControl:sw.isOn];
    [self saveSettingVal];
}

- (IBAction)eFactoryReset:(id)sender
{
    [self showAlert:@"Completed Factory Reset"];
    [[AsReaderBarcodeDevice sharedInstance] doFactoryReset];
}

- (IBAction)switchActionPrefixAndSuffix:(id)sender
{
    UISwitch *switchObject = (UISwitch *)sender;
    NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
    
    switch (switchObject.tag) {
        case 0: // Cumstom Prefix
            [ud setBool:[switchObject isOn] forKey:KEY_SETTING_CUSTOM_PREFIX_ENABLE];
            [ud setObject:[NSString stringWithString:self.customPrefixField.text] forKey:KEY_SETTING_CUSTOM_PREFIX_VALUE];
            [[NSUserDefaults standardUserDefaults] synchronize];
            
            if([switchObject isOn]==YES){
                [[AsReaderBarcodeDevice sharedInstance] setCustomPrefix:self.customPrefixField.text];
            } else {
                [[AsReaderBarcodeDevice sharedInstance] setDisableCustomPrefix];
            }
            break;
            
        case 1: // Cumstom Suffix
            [ud setBool:[switchObject isOn] forKey:KEY_SETTING_CUSTOM_SUFFIX_ENABLE];
            [ud setObject:[NSString stringWithString:self.customSuffixField.text] forKey:KEY_SETTING_CUSTOM_SUFFIX_VALUE];
            [[NSUserDefaults standardUserDefaults] synchronize];
            
            if([switchObject isOn]==YES){
                [[AsReaderBarcodeDevice sharedInstance] setCustomSuffix:self.customSuffixField.text];
            } else {
                [[AsReaderBarcodeDevice sharedInstance] setDisableCustomSuffix];
            }
            break;
            
        case 2: // Symbology Prefix
            [ud setBool:[switchObject isOn] forKey:KEY_SETTING_SYMBOLOGY_PREFIX_ENABLE];
            [[NSUserDefaults standardUserDefaults] synchronize];
            
            if([switchObject isOn]==YES){
                [[AsReaderBarcodeDevice sharedInstance] setSymbologyPrefix];
            } else {
                [[AsReaderBarcodeDevice sharedInstance] setDisableSymbologyPrefix];
            }
            break;
        default:
            break;
    }
}

//Power On Beep
- (IBAction)ePowerOnBeep:(id)sender
{
    [self saveSettingVal];
 
    if([AsReaderInfo sharedInstance].barcodeEngineType == BARCODE_ENGINE_HONEYWELL){
        if( [self.olPowerOnBeep isOn]==YES ){
            [[AsReaderBarcodeDevice sharedInstance] setBarcodeEngineUserCommand:@"BEPPWR1."];
        } else {
            [[AsReaderBarcodeDevice sharedInstance] setBarcodeEngineUserCommand:@"BEPPWR0."];
        }
    }
}

//Beep
- (IBAction)eBeepChanged:(id)sender
{
    [self saveSettingVal];
}

//Illumination
- (IBAction)eIlluminationChanged:(id)sender
{
    [self saveSettingVal];
}

//Vibration
- (IBAction)eVibrationChanged:(id)sender
{
    [self saveSettingVal];
}

//LED
- (IBAction)eLEDChanged:(id)sender
{
    [self saveSettingVal];
}

//Auto Power On /OFF
- (IBAction)eAutoPower:(id)sender
{
    UISwitch *sw = (UISwitch*)sender;
    [[NSUserDefaults standardUserDefaults] setBool:[sw isOn] forKey:@"AutoPowerOn"];
}

//Trigger
- (IBAction)eTriggerDefault:(id)sender
{
    UISwitch *sw = (UISwitch*)sender;
    [[AsReaderCommonDevice sharedInstance] setTriggerMode:[sw isOn]];
    [[NSUserDefaults standardUserDefaults] setBool:[sw isOn] forKey:@"DefaultTriggerOn"];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (IBAction)eChangeUserCommand:(id)sender
{
    UISegmentedControl *seg = (UISegmentedControl *)sender;
    
    if(seg.selectedSegmentIndex == 0) {
        // Manual Mode
        _olUserCommand.text = @"PAPHHF.";
    } else if(seg.selectedSegmentIndex == 1) {
        //Presentation Mode
        _olUserCommand.text = @"TRGMOD3.";
    } else if(seg.selectedSegmentIndex == 2) {
        //1D (All symbol enable + 2D disable
        _olUserCommand.text = @"ALLENA1;QRCENA0.";
    } else if(seg.selectedSegmentIndex == 3) {
        //2D (All symbol disable + 2D enable
        _olUserCommand.text = @"ALLENA0;QRCENA1.";
    }
}



- (IBAction)eSetUserCommand:(id)sender
{
    NSString *strCmd =_olUserCommand.text;
    
    if(strCmd.length<1) {
        [Utils showAlert:@"Error" message:@"Please check User command" view:self];
        return;
    }
    
    [[AsReaderBarcodeDevice sharedInstance] setBarcodeEngineUserCommand:strCmd];
}


#pragma mark - AsreaderBarcodeDeviceDelegate
//----------------------------------------------------------------
- (void)resFactoryRset:(uint8_t)status
{
    dispatch_async(dispatch_get_main_queue(), ^{
        switch (status) {
            case 0x00:
                [self showAlert:@"Res : Doing Reset .... "];
                break;
           
            case 0xFF:
                [self showAlert:@"Completed Factory Reset for Barcode"];
                break;
           
            default:
                break;
        }
    });
}

//- (void)receivedBypassPayload:(NSData *)rawData
//{
//    NSString *ackCommand = [[NSString alloc] initWithData:rawData encoding:NSASCIIStringEncoding];
//    [Utils showAlert:@"Ack from Barcode engine" message:ackCommand view:self];
//}

#pragma mark - UITextField delegate
//----------------------------------------------------------------
- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    [textField resignFirstResponder];
    return YES;
}

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
        return YES;
}

#pragma mark - UITableView delegate
//----------------------------------------------------------------
- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    AsReaderInfo *info = [AsReaderInfo sharedInstance];
    NSString *modelNumber = info.deviceModelNumber;
    
    switch (section) {
        case 0:
            return @"Common functions";
      
        case 1:
            if(!([modelNumber isEqualToString:@"ASR-010D"] ||
                 [modelNumber isEqualToString:@"ASR-010D-V2"] ||
                 [modelNumber isEqualToString:@"ASR-020D"] ||
                 [modelNumber isEqualToString:@"ASR-020D-V2"] ||
                 [modelNumber isEqualToString:@"ASR-022D"] ||
                 [modelNumber isEqualToString:@"ASR-022D-V2U"] ||
                 [modelNumber isEqualToString:@"ASR-030D"] ||
                 [modelNumber isEqualToString:@"ASR-030D-V3"] ||
                 [modelNumber isEqualToString:@"ASR-031D"] ||
                 [modelNumber containsString:@"034B"] ||
                 ([modelNumber containsString:@"ASR-023"] && [info.deviceHardware hasPrefix: @"2."]))) {
                return @"";
            } else {
                return @"Charging control";
            }
            break;
            
        case 2:
            if( !([modelNumber isEqualToString:@"ASR-0230D"]
                 ||[modelNumber isEqualToString:@"ASR-0231D"]
                 ||[modelNumber isEqualToString:@"ASR-0240D"]
                 ||[modelNumber isEqualToString:@"ASR-022D"]
                 ||[modelNumber isEqualToString:@"ASR-022D-V2U"]
                 ||[modelNumber isEqualToString:@"ASR-030D"]
                 ||[modelNumber isEqualToString:@"ASR-030D-V3"]
                 ||[modelNumber isEqualToString:@"ASR-031D"]
                  ||[modelNumber containsString:@"034B"]
                 ||[modelNumber containsString:@"ASR-023"])) {
                return @"";
            } else {
                return @"Default Trigger";
            }
       
        case 3:
            if(!([modelNumber isEqualToString:@"ASR-0230D"]
                 ||[modelNumber isEqualToString:@"ASR-0231D"]
                 ||[modelNumber isEqualToString:@"ASR-0240D"]
                 ||[modelNumber isEqualToString:@"ASR-022D"]
                 ||[modelNumber isEqualToString:@"ASR-022D-V2U"]
                 ||[modelNumber containsString:@"ASR-023"])) {
               return @"";
            } else {
                if( info.currentReaderMode==ReaderModeRfid ) {
                    return @"";
                } else if( info.currentReaderMode==ReaderModeNfc ) {
                    return @"";
                } else {
                    return @"Barcode Control";
                }
            }
            
        default:
            return @"";
    }
}

- (nonnull UITableViewCell *)tableView:(nonnull UITableView *)tableView cellForRowAtIndexPath:(nonnull NSIndexPath *)indexPath { 
    return [super tableView:tableView cellForRowAtIndexPath:indexPath];
}


- (NSInteger)tableView:(nonnull UITableView *)tableView numberOfRowsInSection:(NSInteger)section { 
    return [super tableView:tableView numberOfRowsInSection:section];
}


- (void)plugged:(BOOL)plug { 
    
}

- (void)readerConnected:(int)status { 
    
}

- (void)encodeWithCoder:(nonnull NSCoder *)coder { 
    
}

- (void)traitCollectionDidChange:(nullable UITraitCollection *)previousTraitCollection { 
    
}

- (void)preferredContentSizeDidChangeForChildContentContainer:(nonnull id<UIContentContainer>)container { 
    
}

- (CGSize)sizeForChildContentContainer:(nonnull id<UIContentContainer>)container withParentContainerSize:(CGSize)parentSize { 
    
    return [super sizeForChildContentContainer:container withParentContainerSize:parentSize];
}

- (void)systemLayoutFittingSizeDidChangeForChildContentContainer:(nonnull id<UIContentContainer>)container { 
    
}

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(nonnull id<UIViewControllerTransitionCoordinator>)coordinator { 
    
}

- (void)willTransitionToTraitCollection:(nonnull UITraitCollection *)newCollection withTransitionCoordinator:(nonnull id<UIViewControllerTransitionCoordinator>)coordinator { 
    
}

- (void)didUpdateFocusInContext:(nonnull UIFocusUpdateContext *)context withAnimationCoordinator:(nonnull UIFocusAnimationCoordinator *)coordinator { 
    
}

- (void)setNeedsFocusUpdate { 
    
}

- (BOOL)shouldUpdateFocusInContext:(nonnull UIFocusUpdateContext *)context { 
    
    return [super shouldUpdateFocusInContext:context];
}

- (void)updateFocusIfNeeded { 
    
}

@end
